#!/usr/bin/env python
# -*- coding: euc-jp -*-


##
# @file OutPort.py
# @brief OutPort class
# @date $Date: 2007/09/19$
# @author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
# 
# Copyright (C) 2006-2008
#     Noriaki Ando
#     Task-intelligence Research Group,
#     Intelligent Systems Research Institute,
#     National Institute of
#         Advanced Industrial Science and Technology (AIST), Japan
#     All rights reserved.


from omniORB import any

import OpenRTM

##
# @if jp
# @brief ñѴ
# @else
# @endif
usec_per_sec = 1000000


import time



##
# @if jp
# @class Time
# @brief ִѥ饹
# 
# ꤷͤݻ뤿Υ饹
# 
# @since 0.4.1
# 
# @else
# 
# @endif
class Time:



  ##
  # @if jp
  # @brief 󥹥ȥ饯
  #
  # 󥹥ȥ饯
  #
  # @param self
  #
  # @else
  # @brief Constructor.
  #
  # Constructor.
  #
  # @param self
  #
  # @endif
  def __init__(self):
    global usec_per_sec
    tm = time.time()
    tm_f       = tm - int(tm)     # μФ
    self.sec   = int(tm - tm_f)   # μФ
    self.usec  = int(tm_f * usec_per_sec) # sec -> usec (micro second)



##
# @if jp
#
# @class OutPort
#
# @brief OutPort 饹
# 
# OutPort ѥ饹
#
# @since 0.2.0
#
# @else
# 
# @endif
class OutPort(OpenRTM.OutPortBase):
  """
  """



  ##
  # @if jp
  #
  # @brief 󥹥ȥ饯
  #
  # 󥹥ȥ饯
  #
  # @param self
  # @param name ݡ̾
  # @param value ΥݡȤ˥Хɤǡѿ
  # @param buffer_ Хåե
  #
  # @else
  #
  # @brief Constructor
  #
  # @endif
  def __init__(self, name, value, buffer_):
    OpenRTM.OutPortBase.__init__(self, name)
    self._buffer         = buffer_
    self._value          = value
    self._timeoutTick    = 1000 # timeout tick: 1ms
    self._readBlock      = False
    self._readTimeout    = 0
    self._writeBlock     = False
    self._writeTimeout   = 0
    self._OnWrite        = None
    self._OnWriteConvert = None
    self._OnRead         = None
    self._OnReadConvert  = None
    self._OnOverflow     = None
    self._OnUnderflow    = None


  ##
  # @if jp
  # @brief ǿǡǧ
  #
  # ߤΥХåե֤˳ǼƤǡǿǡǧ롣
  #
  # @param self
  #
  # @return ǿǡǧ
  #            ( true:ǿǡǡϤޤɤ߽ФƤʤ
  #             false:Υǡǡϴɤ߽ФƤ)
  #
  # @else
  #
  # @endif
  def isNew(self):
    return self._buffer.isNew()


  ##
  # @if jp
  #
  # @brief ǡ񤭹
  #
  # ݡȤإǡ񤭹ࡣ
  #
  # - Хåե󥯥 OnWrite åȤƤ硢
  #   OutPort ݻХåե˽񤭹 OnWrite ƤФ롣
  # - OutPort ݻХåեСե򸡽ФǤХåեǤꡢ
  #   ġ񤭹ݤ˥ХåեСե򸡽Ф硢
  #   Хåե󥯥 OnOverflow ƤФ롣
  # - Хåե󥯥 OnWriteConvert åȤƤ硢
  #   Хåե񤭹߻ˡ OnWriteConvert  operator() ͤ
  #   Хåե˽񤭹ޤ롣
  #
  # @param self
  # @param value 񤭹оݥǡ
  #
  # @return 񤭹߽(񤭹:true񤭹߼:false)
  #
  # @else
  #
  # @brief Write data
  #
  # @endif
  # virtual bool write(const DataType& value)
  ##
  # @if jp
  #
  # @brief ǡ񤭹
  #
  # ݡȤإǡ񤭹ࡣ
  # ꤵ줿ͤݡȤ˽񤭹ࡣ
  #
  # @param self
  # @param value 񤭹оݥǡ
  #
  # @return 񤭹߽(񤭹:true񤭹߼:false)
  #
  # @else
  #
  # @endif
  # bool operator<<(DataType& value)
  def write(self, value=None):
    if not value:
      value=self._value

    global usec_per_sec
    
    if self._OnWrite:
      self._OnWrite(value)

    timeout = self._writeTimeout

    tm_pre = Time()

    # blocking and timeout wait
    count = 0
    while self._writeBlock and self._buffer.isFull():
      if self._writeTimeout < 0:
        time.sleep(self._timeoutTick/usec_per_sec)
        continue
      
        
      # timeout wait
      tm_cur = Time()

      sec  = tm_cur.sec - tm_pre.sec
      usec = tm_cur.usec - tm_pre.usec

      timeout -= (sec * usec_per_sec + usec)

      if timeout < 0:
        break
      tm_pre = tm_cur
      time.sleep(self._timeoutTick/usec_per_sec)
      count += 1
      
    if self._buffer.isFull():
      if self._OnOverflow:
        self._OnOverflow(value)
      return False
      
    if not self._OnWriteConvert:
      self._buffer.put(value)
    else:
      self._buffer.put(self._OnWriteConvert(value))

    self.notify()
    return True


  ##
  # @if jp
  #
  # @brief ǡɤ߽Ф
  #
  # DataPort ͤɤ߽Ф
  #
  # - Хåե󥯥 OnRead åȤƤ硢
  #   DataPort ݻХåեɤ߽Ф OnRead ƤФ롣
  # - DataPort ݻХåեե򸡽ФǤХåեǡ
  #   ġɤ߽Фݤ˥Хåեե򸡽Ф硢
  #   Хåե󥯥 OnUnderflow ƤФ롣
  # - Хåե󥯥 OnReadConvert åȤƤ硢
  #   Хåե񤭹߻ˡ OnReadConvert  operator() ͤ
  #   read()ͤȤʤ롣
  # - setReadTimeout() ˤɤ߽ФΥॢȤꤵƤ硢
  #   Хåեե֤ޤǥॢȻ֤Ԥ
  #   OnUnderflow åȤƤФƤӽФ
  #
  # @param self
  # @param value ɤ߽Фǡ
  #
  # @return ɤ߽Ф¹Է(ɤ߽Ф:trueɤ߽Ф:false)
  #
  # @else
  #
  # @brief Read data
  #
  # @endif
  def read(self, value):
    if self._OnRead:
      self._OnRead()

    timeout = self._readTimeout
    tm_pre = Time()

    # blocking and timeout wait
    while self._readBlock and self._buffer.isEmpty():
      if self._readTimeout < 0:
        time.sleep(self._timeoutTick/usec_per_sec)
        continue

      # timeout wait
      tm_cur = Time()
      sec  = tm_cur.sec - tm_pre.sec
      usec = tm_cur.usec - tm_pre.usec
      
      timeout -= (sec * usec_per_sec + usec)
      if timeout < 0:
        break
      tm_pre = tm_cur
      time.sleep(self._timeoutTick/usec_per_sec)

    if self._buffer.isEmpty():
      if self._OnUnderflow:
        value[0] = self._OnUnderflow()
        return False
      else:
        return False

    if not self._OnReadConvert:
      value[0] = self._buffer.get()
      return True
    else:
      value[0] = self._OnReadConvert(self._buffer.get())
      return true

    # never comes here
    return False


  ##
  # @if jp
  #
  # @brief ǡɤ߽ФΥ֥å⡼ɤ
  #
  # ɤ߽ФФƥ֥å⡼ɤꤹ롣
  # ֥å⡼ɤꤷ硢ɤ߽Фǡ뤫ॢ
  # ȯޤǡ read() ᥽åɤθƤӤ֥å롣
  #
  # @param self
  # @param block ֥å⡼ɥե饰
  #
  # @else
  #
  # @brief Set read() block mode
  #
  # @endif
  def setReadBlock(self, block):
    self._readBlock = block


  ##
  # @if jp
  #
  # @brief ǡ񤭹߽Υ֥å⡼ɤ
  #
  # 񤭹߽Фƥ֥å⡼ɤꤹ롣
  # ֥å⡼ɤꤷ硢Хåե˽񤭹ΰ褬Ǥ뤫
  # ॢȤȯޤ write() ᥽åɤθƤӤ֥å롣
  #
  # @param self
  # @param block ֥å⡼ɥե饰
  #
  # @else
  #
  # @brief Set read() block mode
  #
  # @endif
  def setWriteBlock(self, block):
    self._writeBlock = block


  ##
  # @if jp
  #
  # @brief ɤ߽ФΥॢȻ֤
  # 
  # read() ΥॢȻ֤ usec ꤹ롣
  # read() ϥ֥å⡼ɤǤʤФʤʤ
  #
  # @param self
  # @param timeout ॢȻ [usec]
  #
  # @else
  #
  # @brief Set read() timeout
  #
  # @endif
  def setReadTimeout(self, timeout):
    self._readTimeout = timeout


  ##
  # @if jp
  #
  # @brief 񤭹߽ΥॢȻ֤
  # 
  # write() ΥॢȻ֤ usec ꤹ롣
  # write() ϥ֥å⡼ɤǤʤФʤʤ
  #
  # @param self
  # @param timeout ॢȻ [usec]
  #
  # @else
  #
  # @brief Set write() timeout
  #
  # @endif
  def setWriteTimeout(self, timeout):
    self._writeTimeout = timeout


  ##
  # @if jp
  #
  # @brief OnWrite Хå
  #
  # ǡ񤭹ľ˸ƤФ OnWrite Хåե󥯥ꤹ롣
  #
  # @param self
  # @param on_write OnWrite Хåե󥯥
  #
  # @else
  #
  # @brief Set OnWrite callback
  #
  # @endif
  def setOnWrite(self, on_write):
    self._OnWrite = on_write


  ##
  # @if jp
  #
  # @brief OnWriteConvert Хå
  #
  # ǡ񤭹߻˸ƤФ OnWriteConvert Хåե󥯥
  # 롣
  # ΥХåؿν̤񤭹ޤ롣
  # Τ񤭹ߥǡΥե륿󥰤ǽȤʤ롣
  #
  # @param self
  # @param on_wconvert OnWriteConvert Хåե󥯥
  #
  # @else
  #
  # @brief Set OnWriteConvert callback
  #
  # @endif
  def setOnWriteConvert(self, on_wconvert):
    self._OnWriteConvert = on_wconvert


  ##
  # @if jp
  #
  # @brief OnOverflow Хå
  #
  # Хåեեˤǡ񤭹ߤǤʤ˸ƤӽФ OnOverflow
  # Хåե󥯥ꤹ롣
  #
  # @param self
  # @param on_overflow OnOverflow Хåե󥯥
  #
  # @else
  #
  # @brief Set OnOverflow callback
  #
  # @endif
  def setOnOverflow(self, on_overflow):
    self._OnOverflow = on_overflow


  ##
  # @if jp
  #
  # @brief OnRead Хå
  #
  # ǡɤ߽Фľ˸ƤӽФ OnRead Хåե󥯥
  # 롣
  #
  # @param self
  # @param on_read OnRead Хåե󥯥
  #
  # @else
  #
  # @brief Set OnRead callback
  #
  # @endif
  def setOnRead(self, on_read):
    self._OnRead = on_read


  ##
  # @if jp
  #
  # @brief OnReadConvert Хå
  #
  # ǡɤ߽Ф˸ƤФ OnReadConvert Хåե󥯥
  # 롣
  # ΥХåؿν̤ɤ߹ޤ롣
  # Τɤ߹ߥǡΥե륿󥰤ǽȤʤ롣
  #
  # @param self
  # @param on_rconvert OnReadConvert Хåե󥯥
  #
  # @else
  #
  # @brief Set OnReadConvert callback
  #
  # @endif
  def setOnReadConvert(self, on_rconvert):
    self._OnReadConvert = on_rconvert


  ##
  # @if jp
  #
  # @brief OnUnderflow Хå
  #
  # Хåեץƥˤɤ߽Фǡʤ˸ƤӽФ
  # Хåե󥯥 OnUnderflow ꤹ롣
  #
  # @param self
  # @param on_underflow OnUnderflow Хåե󥯥
  #
  # @else
  #
  # @brief Set OnUnderflow callback
  #
  # @endif
  def setOnUnderflow(self, on_underflow):
    self._OnUnderflow = on_underflow


  ##
  # @if jp
  #
  # @brief ǡ̾ѥ᥽å
  #
  # ǡη̾뤿ᡢInPortCorbaProviderƤФ롣
  # 
  # @param self
  #
  # @return ХåեꤵƤǡη̾
  #
  # @else
  #
  # @endif
  def getPortDataType(self):
    val = any.to_any(self._value)
    return str(val.typecode().name())
