#!/usr/bin/env python
# -*- coding: euc-jp -*-

##
# @file OutPortBase.py
# @brief OutPortBase base class
# @date $Date: 2007/09/19 $
# @author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
# 
# Copyright (C) 2003-2008
#     Task-intelligence Research Group,
#     Intelligent Systems Research Institute,
#     National Institute of
#         Advanced Industrial Science and Technology (AIST), Japan
#     All rights reserved.



##
# @if jp
#
# @class OutPortBase
# @brief OutPort 쥯饹
# 
# OutPort μǤ OutPort<T> δ쥯饹
#
# OutPortBase  PublisherBase ϰ Observer ѥƤ롣
# OutPortBase  attach(), detach(), notify() 
# PublisherBase  push()  Observer ѥ˴Ϣ᥽åɤǤ롣
#
# @since 0.2.0
#
# @else
#
# @class OutPortBase
# @brief Output port base class.
#
# The base class of OutPort<T> s which are implementations of OutPort  
#
# @endif
class OutPortBase:
  """
  """



  ##
  # @if jp
  # @brief 󥹥ȥ饯
  #
  # 󥹥ȥ饯
  #
  # @param self
  # @param name ݡ̾
  #
  # @else
  #
  # @brief A constructor of OutPortBase class.
  #
  # Constructor of OutPortBase.
  #
  # @endif
  def __init__(self, name):
    self._name = name
    self._publishers = []


  ##
  # @if jp
  # @brief ǥȥ饯
  #
  # ǥȥ饯
  # Ͽ줿Ƥ Publisher 롣
  #
  # @param self
  #
  # @else
  #
  # @brief destructor
  #
  # Destructor
  #
  # @endif
  def __del__(self):
    for pub in self._publishers:
      del(pub)


  ##
  # @if jp
  # @brief OutPort̾Τμ
  #
  # OutPort̾Τ롣
  #
  # @param self
  #
  # @return ݡ̾
  #
  # @else
  #
  # @brief OutPort's name
  #
  # This operation returns OutPort's name
  #
  # @endif
  def name(self):
    return self._name


  ##
  # @if jp
  # @brief Publisherɲ
  #
  # ꤷPublisherǡȤƥꥹȤκǸɲä롣
  # attach_back() Ʊͤʵǽ
  #
  # @param self
  # @param id_ ꤵ줿Publisher˳ƤID
  # @param publisher ϿоPublisher֥
  #
  # @else
  #
  # @brief Attach a publisher
  #
  # Attach a publisher to observe OutPort.
  #
  # @endif
  def attach(self, id_, publisher):
    self.attach_back(id_, publisher)


  ##
  # @if jp
  # @brief ꥹƬؤPublisherɲ
  #
  # PublisherꥹȤƬɲä롣
  #
  # @param self
  # @param id_ ꤵ줿Publisher˳ƤID
  # @param publisher ϿоPublisher֥
  #
  # @else
  #
  # @brief Attach a publisher
  #
  # Attach a publisher to the head of the Publisher list.
  #
  # @endif
  def attach_front(self, id_, publisher):
    self._publishers.insert(0, self.Publisher(id_, publisher))


  ##
  # @if jp
  # @brief ꥹȺǸؤPublisherɲ
  #
  # PublisherꥹȤκǸɲä롣
  #
  # @param self
  # @param id_ ꤵ줿Publisher˳ƤID
  # @param publisher ϿоPublisher֥
  #
  # @else
  #
  # @brief Attach a publisher
  #
  # Attach a publisher to the taile of the Publisher list.
  #
  # @endif
  def attach_back(self, id_, publisher):
    self._publishers.append(self.Publisher(id_, publisher))


  ##
  # @if jp
  # @brief Publisherκ
  #
  # ꤵ줿 Publisher ǡꥹȤ롣
  #
  # @param self
  # @param id_о Publisher ID
  #
  # @return ϡ Publisher ֥
  #         ꤷ Publisher ¸ߤʤ null
  #
  # @else
  #
  # @brief Detach a publisher
  #
  # Detach a publisher to observe OutPort.
  #
  # @endif
  def detach(self, id_):
    index = -1

    for i in range(len(self._publishers)):
      if id_ == self._publishers[i].id:
        index = i
        break
    if index < 0:
      return None

    pub = self._publishers[index].publisher
    del self._publishers[index]
    return pub


  ##
  # @if jp
  # @brief 
  #
  # ϿƤƤ Publisher ˥ǡΤ롣
  #
  # @param self
  #
  # @else
  #
  # @brief Notify data update
  #
  # This operation notify data update to Publishers
  #
  # @endif
  def notify(self):
    for pub in self._publishers:
      pub.publisher.update()


  ##
  # @if jp
  # @brief Publisher 饹
  # @else
  #
  # @endif
  class Publisher:
    def __init__(self, id_, publisher_):
      self.id = id_
      self.publisher = publisher_


